@@ -1,9 +1,9 @@
 /**************************************************************************
    picpuz         computer graphics jigsaw puzzle
 
-   Copyright 2006, 2007, 2008, 2009, 2010  Michael Cornelison
+   Copyright 2006 2007 2008 2009 2010 2011 2012 Michael Cornelison
    source URL:  kornelix.squarespace.com
-   contact: kornelix@yahoo.de
+   contact: kornelix2@gmail.com
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,26 +22,19 @@
 
 #include "zfuncs.h"
 
-#define gtitle "picpuz v.2.1.3"                                            //  version
+#define gtitle "Picpuz v.2.5"                                              //  version
 #define Tindex(row,col) row * Ncols + col                                  //  map row/col to linear index
 #define drand(seed,range) (drandz(&seed) * range)                          //  random double, 0.0 to 0.9999 * range
 #define lrand(seed,range) (lrandz(&seed) % range)                          //  random integer, 0 to range-1
-
-#define nodither GDK_RGB_DITHER_NONE,0,0
-#define interp GDK_INTERP_HYPER                                            //  image scaling interpolation
-#define solidline GDK_LINE_SOLID,GDK_CAP_BUTT,GDK_JOIN_MITER               //  solid line attributes
-#define dottedline GDK_LINE_DOUBLE_DASH,GDK_CAP_BUTT,GDK_JOIN_MITER        //  dotted line attributes
+#define interp GDK_INTERP_BILINEAR
+#define GDKRGB GDK_COLORSPACE_RGB
 
 GtkWidget      *win1, *vbox1, *dwin1;                                      //  main and drawing window
-GtkWidget      *win2 = 0, *dwin2;                                          //  reference and drawing window
+GtkWidget      *win2, *dwin2;                                              //  reference and drawing window
 GtkWidget      *stbar;
-GdkGC          *gdkgc = 0;                                                 //  graphics context
-GdkPixbuf      *iPixbuf = 0;                                               //  image pixbuff
-GdkPixmap      *iPixmap = 0;                                               //  image pixmap
-GdkColor       black, white, red;
-GdkColormap    *colormap = 0;
-uint           maxcolor = 0xffff;
-int8           dashes[2] = { 3, 3 };
+GdkPixbuf      *iPixbuf;                                                   //  full size image pixbuff
+GdkPixbuf      *wPixbuf;                                                   //  image pixbuf for window
+cairo_t        *mwcr;                                                      //  main window cairo context
 
 timeval     rtime;
 int64       rseed;                                                         //  random number seed
@@ -60,32 +53,31 @@ int         Ntiles = 0;                                                    //  p
 int         Nmoves = 0;                                                    //  tiles moved count
 int         Nhome = 0;                                                     //  tiles at home position
 int         Mstate = 0;                                                    //  mouse, tile select state
+int         linecolor = 0;                                                 //  0/1/2/3 = white/black/red/green
 
-struct t_tileposn {
+struct tileposn_t {
    int      row, col;                                                      //  map tile position
 };
-t_tileposn   *wposn = 0;                                                   //  window position of home tile
-t_tileposn   *hposn = 0;                                                   //  home position of window tile
+tileposn_t   *wposn = 0;                                                   //  window position of home tile
+tileposn_t   *hposn = 0;                                                   //  home position of window tile
 
-void m_gallery();                                                          //  image gallery (thumbnails) window
-void m_open(char * file);                                                  //  open image for new puzzle
-void m_prev();                                                             //  open previous image file
-void m_next();                                                             //  open next image file
+void m_open(char *file);                                                   //  open image for new puzzle
 void m_tile();                                                             //  set new tile size
 void m_mix();                                                              //  mix-up pizzle tiles
 void m_show();                                                             //  show reference image
 void m_save();                                                             //  save puzzle for later
-void m_resm();                                                             //  resume saved puzzle
+void m_resume();                                                           //  resume saved puzzle
 void m_doN(int N);                                                         //  move tiles home
 void m_line();                                                             //  change tile border lines
 void m_quit();                                                             //  exit application
 void m_help();                                                             //  display help file
 
-int  gtkinitfunc(void * data);
-void winpaint();                                                           //  window paint function
+int  gtkinitfunc(void *data);
+void winpaint(GtkWidget *, cairo_t *);                                     //  window paint function
 void menufunc(GtkWidget *, const char *menu);                              //  menu processor
 void destroyfunc();                                                        //  window destroy signal function
 int  puzzle_status();                                                      //  test puzzle status
+void drag_drop(int x, int y, char *file);                                  //  drag/drop event handler   v.2.4
 void init_puzzle(int init);                                                //  initialize puzzle
 void clear_puzzle();                                                       //  release memory, set no puzzle
 void tile_window(int init);                                                //  paint tiles to window
@@ -106,9 +98,8 @@ int main(int argc, char *argv[])
    char        lang[8] = "";
 
    gtk_init(&argc, &argv);                                                 //  GTK command line options
-   zlockInit();
 
-   initz_appfiles("picpuz",null);                                          //  set up app directories
+   zinitapp("picpuz",null);                                                //  set up app directories
 
    for (ii = 1; ii < argc; ii++)                                           //  command line args
    {
@@ -134,17 +125,14 @@ int main(int argc, char *argv[])
    gtk_container_add(GTK_CONTAINER(win1),vbox1);
 
    tbar = create_toolbar(vbox1);                                           //  add toolbar and buttons
-   add_toolbar_button(tbar,ZTX("gallery"),ZTX("image gallery"),"gallery.png",menufunc);
    add_toolbar_button(tbar,ZTX("open"),ZTX("open new image file"),"gtk-open",menufunc);
-   add_toolbar_button(tbar,ZTX("prev"),ZTX("previous image file"),"gtk-go-back",menufunc);
-   add_toolbar_button(tbar,ZTX("next"),ZTX("next image file"),"gtk-go-forward",menufunc);
    add_toolbar_button(tbar,ZTX("tile"),ZTX("set tile size"),"tile.png",menufunc);
    add_toolbar_button(tbar,ZTX("mix"),ZTX("randomize tiles"),"gtk-refresh",menufunc);
    add_toolbar_button(tbar,ZTX("show"),ZTX("show image file"),"show.png",menufunc);
    add_toolbar_button(tbar,ZTX("toolbar::save"),ZTX("save puzzle to a file"),"gtk-save",menufunc);
    add_toolbar_button(tbar,ZTX("resume"),ZTX("resume saved puzzle"),"gtk-open",menufunc);
    add_toolbar_button(tbar,ZTX("do 1"),ZTX("move one tile home"),"piece.png",menufunc);
-   add_toolbar_button(tbar,ZTX("do 8"),ZTX("move ten tiles home"),"piece.png",menufunc);
+   add_toolbar_button(tbar,ZTX("do 8"),ZTX("move eight tiles home"),"piece.png",menufunc);
    add_toolbar_button(tbar,ZTX("line"),ZTX("change tile border line"),"line.png",menufunc);
    add_toolbar_button(tbar,ZTX("quit"),ZTX("quit picpuz"),"gtk-quit",menufunc);
    add_toolbar_button(tbar,ZTX("help"),ZTX("view help document"),"gtk-help",menufunc);
@@ -156,28 +144,16 @@ int main(int argc, char *argv[])
 
    gtk_widget_add_events(dwin1,GDK_BUTTON_PRESS_MASK);                     //  connect signals
    gtk_widget_add_events(dwin1,GDK_BUTTON_RELEASE_MASK);
-   G_SIGNAL(dwin1,"button-press-event",mouse_event,0)
-   G_SIGNAL(dwin1,"button-release-event",mouse_event,0)
-   G_SIGNAL(dwin1,"expose-event",winpaint,0)
-   G_SIGNAL(win1,"destroy",destroyfunc,0)
+   G_SIGNAL(dwin1,"button-press-event",mouse_event,0);
+   G_SIGNAL(dwin1,"button-release-event",mouse_event,0);
+   G_SIGNAL(dwin1,"draw",winpaint,0);                                      //  gtk3
+   G_SIGNAL(win1,"destroy",destroyfunc,0);
+   
+   drag_drop_connect(dwin1,drag_drop);                                     //  connect drag-drop event      v.2.4
 
    gtk_widget_show_all(win1);                                              //  show all widgets
 
-   gtk_init_add((GtkFunction) gtkinitfunc,0);                              //  setup initz. call from gtk_main()
-
-   gdkgc = gdk_gc_new(dwin1->window);                                      //  initz. graphics context
-   black.red = black.green = black.blue = 0;
-   white.red = white.green = white.blue = maxcolor;
-   red.red = maxcolor; red.green = red.blue = 0;
-   colormap = gtk_widget_get_colormap(dwin1);
-   gdk_rgb_find_color(colormap,&black);
-   gdk_rgb_find_color(colormap,&white);
-   gdk_rgb_find_color(colormap,&red);
-   gdk_gc_set_foreground(gdkgc,&black);
-   gdk_gc_set_background(gdkgc,&white);
-   gdk_gc_set_dashes(gdkgc,0,(gint8 *) dashes,2);
-   gdk_gc_set_fill(gdkgc,GDK_SOLID);
-   gdk_gc_set_line_attributes(gdkgc,1,solidline);
+   g_timeout_add(0,gtkinitfunc,0);                                         //  setup initz. call         gtk3
 
    gtk_main();                                                             //  process window events
    return 0;
@@ -187,7 +163,7 @@ int main(int argc, char *argv[])
 //  initial function called from gtk_main() at startup
 //  (must be done after gtk_main() loop has started)
 
-int gtkinitfunc(void * data)
+int gtkinitfunc(void *)
 {
    char     *pp;
 
@@ -196,9 +172,6 @@ int gtkinitfunc(void * data)
 
    if (! *imagedirk) load_imagedirk();                                     //  get image directory   v.1.8
 
-   image_gallery(imagedirk,"init");                                        //  initz. image file list
-// m_gallery();                                                            //  no open image gallery    v.2.1
-
    if (*clfile) {                                                          //  command line image file
       if (*clfile != '/') {
          pp = zmalloc(strlen(imagedirk)+strlen(clfile)+5);                 //  clfile is relative to imagedirk
@@ -217,21 +190,12 @@ int gtkinitfunc(void * data)
 
 //  paint window when created, exposed, or resized
 
-void winpaint()
+void winpaint(GtkWidget *, cairo_t *cr)
 {
-   GdkRectangle   rect;
-
-   winW = dwin1->allocation.width;                                         //  window size
-   winH = dwin1->allocation.height;
-
-   rect.x = rect.y = 0;
-   rect.width = winW;
-   rect.height = winH;
-   
-// gdk_window_begin_paint_rect(dwin1->window,&rect);                       //  does not help   v.2.1.2
+   mwcr = cr;                                                              //  gtk3
+   cairo_set_line_width(mwcr,1);
    if (Ntiles) tile_window(0);                                             //  paint tiles
-// gdk_window_end_paint(dwin1->window);
-
+   mwcr = 0;
    return;
 }
 
@@ -240,15 +204,12 @@ void winpaint()
 
 void menufunc(GtkWidget *, const char *menu)
 {
-   if (strEqu(menu,ZTX("gallery")))  m_gallery();
    if (strEqu(menu,ZTX("open"))) m_open(0);
-   if (strEqu(menu,ZTX("prev"))) m_prev();
-   if (strEqu(menu,ZTX("next"))) m_next();
    if (strEqu(menu,ZTX("tile"))) m_tile();
    if (strEqu(menu,ZTX("mix"))) m_mix();
    if (strEqu(menu,ZTX("show"))) m_show();
-   if (strEqu(menu,ZTX("toolbar::save"))) m_save();                        //  bugfix  v.1.9.3
-   if (strEqu(menu,ZTX("resume"))) m_resm();
+   if (strEqu(menu,ZTX("toolbar::save"))) m_save();
+   if (strEqu(menu,ZTX("resume"))) m_resume();
    if (strEqu(menu,ZTX("do 1"))) m_doN(1);
    if (strEqu(menu,ZTX("do 8"))) m_doN(8);
    if (strEqu(menu,ZTX("line"))) m_line();
@@ -257,41 +218,23 @@ void menufunc(GtkWidget *, const char *menu)
 }
 
 
-//  display an image gallery (thumbnails) in a separate window
-
-void m_gallery()
-{
-   if (imagefile) image_gallery(imagefile,"paint1",0,m_open);              //  v.1.8
-   else {
-      image_gallery(imagedirk,"init",0,m_open);
-      image_gallery(0,"paint1");
-   }
-   return;
-}
-
-
 //  create a new puzzle from an image file
 
-void m_open(char * filez)
+void m_open(char *file)
 {
-   char        *newfile, *pp;
+   char        *newfile = 0, *pp;
    
-   if (filez && strEqu(filez,"F1")) return;                                //  context help     v.2.1
+   if (file && strEqu(file,"F1")) return;                                  //  context help     v.2.1
    
    if (puzzle_status()) return;                                            //  do not discard
    clear_puzzle();
    
-   newfile = filez;
+   if (file) newfile = strdupz(file);
    if (! newfile) {
-      newfile = zgetfile(ZTX("select image file"),imagedirk,"open");
+      newfile = zgetfile1(ZTX("select image file"),"open",imagedirk);
       if (! newfile) return;
    }
 
-   if (! samedirk(imagefile,newfile)) {                                    //  if directory changed,
-      image_gallery(newfile,"init");                                       //    initz. image file list
-      image_gallery(0,"paint2");                                           //      and image gallery window
-   }
-
    if (imagefile) zfree(imagefile);
    imagefile = newfile;
 
@@ -301,27 +244,15 @@ void m_open(char * filez)
 
    if (win2) gtk_widget_destroy(win2);                                     //  reference window obsolete    v.2.1.3
    init_puzzle(1);                                                         //  initz. main window
-// gtk_window_present(GTK_WINDOW(win1));                                   //  no longer needed             v.2.1.3
    return;
 }
 
-void m_prev()                                                              //  previous image file
+
+//  drag-drap enent. open the dropped file.
+
+void drag_drop(int x, int y, char *file)                                   //  v.2.4
 {
-   char     *filez;
-
-   if (! imagefile) return;
-   filez = image_gallery(imagefile,"prev");
-   if (image_file_type(filez) == 2) m_open(filez);
-   return;
-}
-
-void m_next()                                                              //  next image file
-{
-   char     *filez;
-
-   if (! imagefile) return;
-   filez = image_gallery(imagefile,"next");
-   if (image_file_type(filez) == 2) m_open(filez);
+   m_open(file);
    return;
 }
 
@@ -362,16 +293,13 @@ void m_tile()
 
 void m_mix()
 {
-   int      row1, col1, row2, col2, ii;
+   int      row1, col1, row2, col2;
 
    if (puzzle_status()) return;                                            //  do not discard
 
    for (row1 = 0; row1 < Nrows; row1++)                                    //  randomize tile positions
    for (col1 = 0; col1 < Ncols; col1++)
    {
-      ii = Tindex(row1,col1);
-//    if (wposn[ii].row != row1) continue;                                 //  mix anyway       v.2.1.3
-//    if (wposn[ii].col != col1) continue;
       row2 = lrand(rseed,Nrows);
       col2 = lrand(rseed,Ncols);
       swap_tiles(row1,col1,row2,col2);
@@ -388,32 +316,31 @@ void m_mix()
 
 void m_show()
 {
-   void win2_paint();
+   void win2_paint(GtkWidget *, cairo_t *);
    void win2_destroy();
 
    if (! iPixbuf) return;                                                  //  no image
 
    if (win2) {
-      win2_paint();
+      gtk_widget_queue_draw(dwin2);
       return;
    }
 
    win2 = gtk_window_new(GTK_WINDOW_TOPLEVEL);                             //  create window
    gtk_window_set_position(GTK_WINDOW(win2),GTK_WIN_POS_CENTER);
    gtk_window_set_default_size(GTK_WINDOW(win2),400,300);
-// gtk_window_set_transient_for(GTK_WINDOW(win2),GTK_WINDOW(win1));        //  v.2.1.2
    
    dwin2 = gtk_drawing_area_new();                                         //  add drawing window
    gtk_container_add(GTK_CONTAINER(win2),dwin2);
 
-   G_SIGNAL(win2,"destroy",win2_destroy,0)                                 //  connect events
-   G_SIGNAL(dwin2,"expose-event",win2_paint,0)
+   G_SIGNAL(win2,"destroy",win2_destroy,0);                                //  connect events
+   G_SIGNAL(dwin2,"draw",win2_paint,0);
 
    gtk_widget_show_all(win2);                                              //  show all widgets
    return;
 }
 
-void win2_paint()
+void win2_paint(GtkWidget *, cairo_t *cr)
 {
    GdkPixbuf   *refPixbuf;
    int         x, y, winx, winy;
@@ -421,10 +348,9 @@ void win2_paint()
    if (! iPixbuf) return;
 
    gtk_window_set_title(GTK_WINDOW(win2),pname);
-   gdk_window_clear(dwin2->window);
 
-   winx = dwin2->allocation.width;                                         //  curr. window size
-   winy = dwin2->allocation.height;
+   winx = gtk_widget_get_allocated_width(dwin2);                           //  curr. window size
+   winy = gtk_widget_get_allocated_height(dwin2);
    
    x = int(1.0 * winy * imageW / imageH);                                  //  preserve X/Y ratio
    if (winx > x) winx = x;
@@ -432,9 +358,10 @@ void win2_paint()
    if (winy > y) winy = y;
    
    refPixbuf = gdk_pixbuf_scale_simple(iPixbuf,winx,winy,interp);          //  scale image to window
-
-   gdk_draw_pixbuf(dwin2->window,0,refPixbuf,0,0,0,0,-1,-1,nodither);      //  paint window
+   gdk_cairo_set_source_pixbuf(cr,refPixbuf,0,0);                          //  gtk3 
+   cairo_paint(cr);
    g_object_unref(refPixbuf);
+
    return;
 }
 
@@ -459,12 +386,12 @@ void m_save()
    *savefile = 0;                                                          //  /home/user/.picpuz/pname.puz
    strncatv(savefile,999,get_zuserdir(),"/",pname,".puz",null);
 
-   sfile = zgetfile(ZTX("save puzzle to a file"),savefile,"save");
+   sfile = zgetfile1(ZTX("save puzzle to a file"),"save",savefile);
    if (! sfile) return;
 
    fid = fopen(sfile,"w");
    if (! fid) { 
-      zmessageACK(GTK_WINDOW(win1),ZTX("cannot open: %s"),sfile);
+      zmessageACK(win1,ZTX("cannot open: %s"),sfile);
       zfree(sfile);
       return;
    }
@@ -491,7 +418,7 @@ void m_save()
 
 //  resume a saved puzzle
 
-void m_resm()
+void m_resume()
 {
    FILE        *fid;
    int         stat, row1, col1, row2, col2, ii;
@@ -500,12 +427,12 @@ void m_resm()
    if (puzzle_status()) return;                                            //  do not discard
    clear_puzzle();
 
-   newfile = zgetfile(ZTX("load puzzle from file"),get_zuserdir(),"open");
+   newfile = zgetfile1(ZTX("load puzzle from file"),"open",get_zuserdir());
    if (! newfile) return;
 
    fid = fopen(newfile,"r");
    if (! fid) {
-      zmessageACK(GTK_WINDOW(win1),ZTX("cannot open: %s"),newfile);
+      zmessageACK(win1,ZTX("cannot open: %s"),newfile);
       zfree(newfile);
       return;
    }
@@ -515,11 +442,6 @@ void m_resm()
    pp = fgets_trim(newfile,maxfcc-1,fid,1);                                //  read image file name
    if (! pp) goto badfile;
 
-   if (! samedirk(imagefile,newfile)) {                                    //  if directory changed,
-      image_gallery(newfile,"init");                                       //    initz. image file list
-      image_gallery(0,"paint2");                                           //      and image gallery window
-   }
-
    if (imagefile) zfree(imagefile);
    imagefile = newfile;
    
@@ -531,7 +453,7 @@ void m_resm()
    if (Ntiles != Nrows * Ncols) goto badfile;
 
    if (wposn) delete [] wposn;
-   wposn = new t_tileposn[Ntiles];   
+   wposn = new tileposn_t[Ntiles];   
    
    for (row1 = 0; row1 < Nrows; row1++) {                                  //  read tile position data
       for (col1 = 0; col1 < Ncols; col1++) {
@@ -552,7 +474,7 @@ void m_resm()
    
 badfile:
    fclose(fid);
-   zmessageACK(GTK_WINDOW(win1),ZTX("saved puzzle file is not valid"));
+   zmessageACK(win1,ZTX("saved puzzle file is not valid"));
    clear_puzzle();
    return;
 }
@@ -597,39 +519,13 @@ mdone:
 }
 
 
-//  chage tile borderline style
+//  chage tile borderline color                                            //  gtk3
 
 void m_line()
 {
-   static int  linestyle = 0;
-
-   linestyle++;
-   if (linestyle > 3) linestyle = 0;
-
-   switch (linestyle) {
-   
-   case 0: 
-      gdk_gc_set_foreground(gdkgc,&black);
-      gdk_gc_set_line_attributes(gdkgc,1,solidline);
-      break;
-
-   case 1: 
-      gdk_gc_set_foreground(gdkgc,&white);
-      gdk_gc_set_line_attributes(gdkgc,1,solidline);
-      break;
-
-   case 2: 
-      gdk_gc_set_foreground(gdkgc,&black);
-      gdk_gc_set_line_attributes(gdkgc,1,dottedline);
-      break;
-
-   case 3: 
-      gdk_gc_set_foreground(gdkgc,&red);                                   //  v.2.0
-      gdk_gc_set_line_attributes(gdkgc,1,solidline);
-      break;
-   }
-
-   winpaint();
+   linecolor++;
+   if (linecolor > 3) linecolor = 0;
+   gtk_widget_queue_draw(dwin1);
    return;
 }
 
@@ -664,7 +560,7 @@ int puzzle_status()
    if (! Ntiles) return 0;                                                 //  no puzzle
    if (! Nmoves) return 0;                                                 //  no new moves
    if (Nhome == Ntiles) return 0;                                          //  puzzle done
-   if (zmessageYN(GTK_WINDOW(win1),ZTX("DISCARD unfinished puzzle?")))     //  OK to discard
+   if (zmessageYN(win1,ZTX("DISCARD unfinished puzzle?")))                 //  OK to discard
       return 0;
    return 1;                                                               //  keep
 }
@@ -697,7 +593,7 @@ void init_puzzle(int newp)
    if (iPixbuf) g_object_unref(iPixbuf);
    iPixbuf = gdk_pixbuf_new_from_file(imagefile,&gerror);                  //  create pixbuf from image file
    if (!iPixbuf) {
-      zmessageACK(GTK_WINDOW(win1),ZTX("image type not recognized:\n %s"),imagefile);
+      zmessageACK(win1,ZTX("image type not recognized:\n %s"),imagefile);
       clear_puzzle();
       return;
    }
@@ -706,12 +602,11 @@ void init_puzzle(int newp)
    imageH = gdk_pixbuf_get_height(iPixbuf);
 
    if (imageW < 300 || imageH < 300) {
-      zmessageACK(GTK_WINDOW(win1),ZTX("image too small, please select another"));
+      zmessageACK(win1,ZTX("image too small, please select another"));
       clear_puzzle();
       return;
    }
 
-// if (win2) win2_paint();                                                 //  no longer needed             v.2.1.3
    tile_window(newp);                                                      //  paint tiles on main window
 
    return;
@@ -725,7 +620,6 @@ void clear_puzzle()
    if (iPixbuf) g_object_unref(iPixbuf);
    iPixbuf = 0;
    Ntiles = Nmoves = Nhome = 0;
-   gdk_window_clear(dwin1->window);
    stbar_update();
    return;
 }
@@ -736,15 +630,12 @@ void clear_puzzle()
 
 void tile_window(int newp)
 {
-   GdkPixbuf   *wPixbuf;
    int         x1, y1, ii, row, col;
    
-   gdk_window_clear(dwin1->window);                                        //  blank window
-
    if (! iPixbuf) return;
-
-   winW = dwin1->allocation.width;                                         //  curr. window size
-   winH = dwin1->allocation.height;
+   
+   winW = gtk_widget_get_allocated_width(dwin1);                           //  window size
+   winH = gtk_widget_get_allocated_height(dwin1);
    
    winW = winW - 4;                                                        //  to keep margins visible
    winH = winH - 4;
@@ -762,7 +653,7 @@ void tile_window(int newp)
       Nhome = Ntiles;
 
       if (wposn) delete [] wposn;                                          //  new window position map
-      wposn = new t_tileposn[Ntiles];   
+      wposn = new tileposn_t[Ntiles];   
 
       for (row = 0; row < Nrows; row++)
       for (col = 0; col < Ncols; col++)
@@ -774,7 +665,7 @@ void tile_window(int newp)
    }
 
    if (hposn) delete [] hposn;                                             //  new home position map
-   hposn = new t_tileposn[Ntiles];   
+   hposn = new tileposn_t[Ntiles];   
 
    for (row = 0; row < Nrows; row++)                                       //  initialize home position map
    for (col = 0; col < Ncols; col++)                                       //    from window position map
@@ -791,17 +682,24 @@ void tile_window(int newp)
    winW = Ncols * tileW;                                                   //  synch. window to tile size
    winH = Nrows * tileH;
 
+   if (wPixbuf) g_object_unref(wPixbuf);                                   //  gtk3
    wPixbuf = gdk_pixbuf_scale_simple(iPixbuf,winW,winH,interp);            //  scale image to window size
-
-   if (iPixmap) g_object_unref(iPixmap);                                   //  allocate new pixmap
-   iPixmap = gdk_pixmap_new(dwin1->window,winW,winH,-1);
-   gdk_draw_pixbuf(iPixmap,0,wPixbuf,0,0,0,0,-1,-1,nodither);              //  copy scaled image to pixmap
-   g_object_unref(wPixbuf);
-
-   for (row = 0; row < Nrows; row++)                                       //  draw tile pixmaps on window
-   for (col = 0; col < Ncols; col++) 
-      draw_tile(row,col);
    
+   if (mwcr) {
+      for (row = 0; row < Nrows; row++)                                    //  draw tile pixmaps on window
+      for (col = 0; col < Ncols; col++) 
+         draw_tile(row,col);
+   }
+   else {
+      mwcr = gdk_cairo_create(gtk_widget_get_window(dwin1));               //  gtk3
+      cairo_set_line_width(mwcr,1);
+      for (row = 0; row < Nrows; row++)
+      for (col = 0; col < Ncols; col++) 
+         draw_tile(row,col);
+      cairo_destroy(mwcr);
+      mwcr = 0;
+   }
+
    Mstate = 0;                                                             //  no tile selected
    stbar_update();                                                         //  update status bar
    return;
@@ -898,8 +796,12 @@ void swap2(int row1, int col1, int row2, int col2)
    SWAP(wposn[ii1].row,wposn[ii2].row)
    SWAP(wposn[ii1].col,wposn[ii2].col)
    
+   mwcr = gdk_cairo_create(gtk_widget_get_window(dwin1));                  //  gtk3
+   cairo_set_line_width(mwcr,1);
    draw_tile(row1,col1);                                                   //  draw tiles at new positions
    draw_tile(row2,col2);
+   cairo_destroy(mwcr);
+   mwcr = 0;
 
    Nmoves++;                                                               //  incr. move count
 
@@ -1004,9 +906,9 @@ void swap3(int row1, int col1, int row2, int col2)
 
 void draw_tile(int row, int col)
 {
-   void     draw_body(int row, int col);
-   void     draw_lobe(int row, int col);
-   void     draw_hole(int row, int col);
+   void  draw_body(int row, int col);
+   void  draw_lobe(int row, int col);
+   void  draw_hole(int row, int col);
    
    if (col > 0) {
       draw_body(row,col-1);                                                //  refresh tile to left
@@ -1026,21 +928,35 @@ void draw_tile(int row, int col)
 
 void draw_body(int row2, int col2)
 {
-   int      ii, row1, col1;
-   int      x1, y1, x2, y2;
-   
-   ii = Tindex(row2,col2);                                                 //  get home position
+   int         ii, row1, col1;
+   int         x1, y1, x2, y2, rs;
+   uint8       *ptile;
+   GdkPixbuf   *tPixbuf;                                                   //  tile pixbuf
+
+   ii = Tindex(row2,col2);                                                 //  tile home position
    row1 = hposn[ii].row;
    col1 = hposn[ii].col;
    
-   x1 = col1 * tileW;                                                      //  draw tile pixmap to its
-   y1 = row1 * tileH;                                                      //    current window position
-   x2 = col2 * tileW;
+   x1 = col1 * tileW;                                                      //  position in image
+   y1 = row1 * tileH;
+   x2 = col2 * tileW;                                                      //  position in window
    y2 = row2 * tileH;
-   gdk_draw_drawable(dwin1->window,gdkgc,iPixmap,x1,y1,x2,y2,tileW,tileH);
 
-   gdk_draw_rectangle(dwin1->window,gdkgc,0,x2,y2,tileW,tileH);            //  draw boundary line
+   rs = gdk_pixbuf_get_rowstride(wPixbuf);
+   ptile = gdk_pixbuf_get_pixels(wPixbuf) + y1 * rs + 3 * x1;              //  draw tile        gtk3
+   tPixbuf = gdk_pixbuf_new_from_data(ptile,GDKRGB,0,8,tileW,tileH,rs,0,0);
+   gdk_cairo_set_source_pixbuf(mwcr,tPixbuf,x2,y2);
+   cairo_paint(mwcr);
+   g_object_unref(tPixbuf);
    
+   if (linecolor == 0) cairo_set_source_rgb(mwcr,1,1,1);                   //  draw tile outline   gtk3
+   if (linecolor == 1) cairo_set_source_rgb(mwcr,0,0,0);
+   if (linecolor == 2) cairo_set_source_rgb(mwcr,1,0,0);
+   if (linecolor == 3) cairo_set_source_rgb(mwcr,0,1,0);
+
+   cairo_rectangle(mwcr,x2,y2,tileW,tileH);
+   cairo_stroke(mwcr);
+
    return;
 }
 
@@ -1048,9 +964,11 @@ void draw_body(int row2, int col2)
 
 void draw_lobe(int row2, int col2)
 {
-   int      ii, row1, col1;
-   int      x1, y1, x2, y2, px, py, pw, ph;
-   int64    seed;
+   int         ii, row1, col1, rs;
+   int         x1, y1, x2, y2, px, py, pw, ph;
+   int64       seed;
+   uint8       *plobe;
+   GdkPixbuf   *lPixbuf;                                                   //  lobe pixbuf
    
    if (col2 == 0) return;                                                  //  window position on left edge
 
@@ -1064,15 +982,28 @@ void draw_lobe(int row2, int col2)
    py = int(drand(seed,0.6) * tileH + 0.1 * tileH);
    pw = int(0.2 * tileW + 0.5);
    ph = int(0.2 * tileH + 0.5);
-   x1 = col1 * tileW - px;
+   x1 = col1 * tileW - px;                                                 //  position in image
    y1 = row1 * tileH + py;
-   x2 = col2 * tileW - px;
+   x2 = col2 * tileW - px;                                                 //  position in window
    y2 = row2 * tileH + py;
-   gdk_draw_drawable(dwin1->window,gdkgc,iPixmap,x1,y1,x2,y2,pw+1,ph);     //  draw lobe
 
-   gdk_draw_line(dwin1->window,gdkgc,x2,y2,x2+pw,y2);                      //  draw boundary line
-   gdk_draw_line(dwin1->window,gdkgc,x2,y2,x2,y2+ph);
-   gdk_draw_line(dwin1->window,gdkgc,x2,y2+ph,x2+pw,y2+ph);
+   rs = gdk_pixbuf_get_rowstride(wPixbuf);
+   plobe = gdk_pixbuf_get_pixels(wPixbuf) + y1 * rs + 3 * x1;              //  draw lobe        gtk3
+   lPixbuf = gdk_pixbuf_new_from_data(plobe,GDKRGB,0,8,pw,ph,rs,0,0);
+   gdk_cairo_set_source_pixbuf(mwcr,lPixbuf,x2,y2);
+   cairo_paint(mwcr);
+   g_object_unref(lPixbuf);
+
+   if (linecolor == 0) cairo_set_source_rgb(mwcr,1,1,1);
+   if (linecolor == 1) cairo_set_source_rgb(mwcr,0,0,0);
+   if (linecolor == 2) cairo_set_source_rgb(mwcr,1,0,0);
+   if (linecolor == 3) cairo_set_source_rgb(mwcr,0,1,0);
+
+   cairo_move_to(mwcr,x2+pw,y2);                                           //  draw lobe outline    gtk3
+   cairo_line_to(mwcr,x2,y2);
+   cairo_line_to(mwcr,x2,y2+ph);
+   cairo_line_to(mwcr,x2+pw,y2+ph);
+   cairo_stroke(mwcr);
 
    return;
 }
@@ -1095,10 +1026,13 @@ void draw_hole(int row2, int col2)
    py = int(drand(seed,0.6) * tileH + 0.1 * tileH);
    pw = int(0.2 * tileW + 0.5);
    ph = int(0.2 * tileH + 0.5);
-   x2 = col2 * tileW + tileW - px;
+   x2 = col2 * tileW + tileW - px;                                         //  hole position in window
    y2 = row2 * tileH + py;
-   gdk_draw_rectangle(dwin1->window,gdkgc,1,x2,y2,pw,ph);
    
+   cairo_set_source_rgb(mwcr,0,0,0);                                       //  draw hole rectangle       gtk3
+   cairo_rectangle(mwcr,x2,y2,pw,ph);
+   cairo_fill(mwcr);
+
    return;
 }   
 
@@ -1122,11 +1056,12 @@ void stbar_update()
 void save_imagedirk()
 {
    char     command[maxfcc+100];
-   int      ignore;
+   int      err;
    
    snprintf(command,maxfcc+99,"echo %s > %s/image_directory",              //  save imagedirk to file
                                     imagedirk, get_zuserdir());
-   ignore = system(command);
+   err = system(command);
+   if (err) printf("error: %s \n",wstrerror(err));   
    return;
 }
 
