@@ -1,9 +1,9 @@
 /**************************************************************************
    zfuncs.h      include file for zfuncs functions
 
-   Copyright 2006, 2007, 2008, 2009, 2010  Michael Cornelison
+   Copyright 2006 2007 2008 2009 2010 2011 2012  Michael Cornelison
    source URL:  kornelix.squarespace.com
-   contact: kornelix@yahoo.de
+   contact: kornelix2@googlemail.com
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,10 +20,12 @@
 
 ***************************************************************************/
 
-//     zfuncs.h   version  v.3.5
+//     zfuncs.h   version  v.5.1    GTK3 conversion
 
 #include <sys/time.h>
 #include <sys/stat.h>
+#include <sys/resource.h>
+#include <sys/file.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -36,7 +38,7 @@
 #include <signal.h>
 #include <execinfo.h>
 #include <locale.h>
-#include <sys/inotify.h>
+#include <gtk/gtk.h>
 
 #define  int8   char                                                       //  number types
 #define  int16  short
@@ -46,9 +48,9 @@
 #define  uint16  unsigned short
 #define  uint32  unsigned long
 #define  uint64  unsigned long long
+#define  dub  double
 #define  uchar  unsigned char
 #define  cchar  const char
-#define  dub  double
 
 #define  wstrerror(err) strerror(WEXITSTATUS(err))                         //  get text status for child process
 
@@ -66,28 +68,42 @@
 
 //  system functions ======================================================
 
-void appcrash(cchar * pMess, ... );                                        //  fatal exit - works like printf
-void zappcrash(cchar *pMessage, ...);                                      //  crash with popup message in xterm window
 void apppause();                                                           //  output message and wait for user
 void apppause(cchar * pMess, ... );                                        //  same, works like printf
+void zappcrash(cchar *pMessage, ...);                                      //  crash with popup message in text window
 void catch_signals();                                                      //  catch signals and do backtrace dump
+void beroot(int argc = 0, char *argv[] = 0);                               //  restart image as root if password OK
+
 double get_seconds();                                                      //  seconds since 2000.01.01
 void start_timer(double &time0);                                           //  start a timer
 double get_timer(double &time0);                                           //  get elapsed time in seconds
 void start_CPUtimer(double &time0);                                        //  start a process CPU timer
 double get_CPUtimer(double &time0);                                        //  get elapsed CPU time, seconds
-double CPUtime();                                                          //  get elapsed process CPU time used
+double CPUtime();                                                          //  get elapsed process CPU time for main()
+double CPUtime2();                                                         //   " include all threads 
+double jobtime();                                                          //   " include all threads + subprocesses
+int  parseprocfile(cchar *pfile, cchar *pname, double *value, ...);        //  get data from /proc file
+int  parseprocrec(char *prec, int field, double *value, ...);              //  get data from /proc file record
+void zsleep(double dsecs);                                                 //  sleep specified seconds
+int global_lock(cchar *lockname);                                          //  obtain exclusive lock, multi-process
+int global_unlock(int fd);                                                 //  release the lock
+
 void start_detached_thread(void * tfunc(void *), void * arg);              //  start detached thread function
 void synch_threads(int NT = 0);                                            //  synchronize NT threads
+
 int  zget_locked(int &param);                                              //  lock and get multi-thread parameter
 void zput_locked(int &param, int value);                                   //  put and unlock multi-thread parameter
 int  zadd_locked(int &param, int incr);                                    //  increment multi-thread parameter
-void zsleep(double dsecs);                                                 //  sleep specified seconds
+
+void gate_threads_enter();                                                 //  allow only one thread at a time within
+void gate_threads_leave();                                                 //    the time betwen enter() and leave()
+
 char * zmalloc(size_t bytes, cchar *tag = 0);                              //  malloc() with auto crash and log option
 void zfree(void *buff);                                                    //  free() with log option
-void zmalloc_log(int count);                                               //  turn zmalloc() log on/off       v.3.5
-void zmalloc_report();                                                     //  print total memory allocated    v.3.5
-char * command_output(int &contx, cchar *command, ...);                    //  get shell command output        v.2.3
+void zmalloc_log(int count);                                               //  turn zmalloc() log on/off
+void zmalloc_report();                                                     //  print memory allocation report
+
+char * command_output(int &contx, cchar *command, ...);                    //  get shell command output
 int command_status(int contx);                                             //  get exit status of command
 int signalProc(cchar * pname, cchar * signal);                             //  pause/resume/kill subprocess
 int runroot(cchar *sucomm, cchar *command);                                //  run command as root via su or sudo
@@ -95,9 +111,9 @@ int checkinstall(cchar *prog, ...);                                        //  c
 char * fgets_trim(char * buff, int maxcc, FILE *, int bf = 0);             //  fgets + trim trailing \n \r (blanks)
 int samedirk(cchar *file1, cchar *file2);                                  //  returns 1 if files in same directory
 int parsefile(cchar *path, char **dir, char **file, char **ext);           //  parse a filespec
-int zmondirk(cchar *action, cchar *dirk, char **file);                     //  monitor a directory for changes   v.2.16
 
 //  measure CPU time spent in a function or code block within a function
+
 extern volatile double   cpu_profile_timer;                                //  internal data tables
 extern volatile double   cpu_profile_table[100];
 extern volatile double   cpu_profile_elapsed;
@@ -127,6 +143,7 @@ void strnPad(char *dest, cchar *source, int cc);                           //  s
 int  strTrim(char *dest, cchar *source);                                   //  remove trailing blanks
 int  strTrim(char *string);                                                //  remove trailing blanks
 int  strTrim2(char *dest, cchar *source);                                  //  remove leading and trailing blanks
+int  strTrim2(char *string);                                               //  remove leading and trailing blanks
 int  strCompress(char *dest, cchar *source);                               //  remove all blanks incl. imbedded
 int  strCompress(char *string);                                            //  remove all blanks
 int  strncatv(char *dest, int maxcc, cchar *source, ...);                  //  catenate strings (last = 0L) 
@@ -197,7 +214,8 @@ cchar * SearchWild(cchar *wpath, int &flag);                               //  w
 
 //  search and sort functions =============================================
 
-int bsearch(int element, int nn, int list[]);                              //  binary search sorted list[nn]
+int bsearch(int seekint, int nn, int list[]);                              //  binary search sorted list[nn]
+int bsearch(char *seekrec, char *allrecs, int recl, int nrecs);            //  binary search sorted records
 
 typedef int HeapSortUcomp(cchar * rec1, cchar * rec2);                     //  return -1/0/+1 if rec1 </=/> rec2
 void HeapSort(int vv[], int nn);                                           //  Heap Sort - integer
@@ -225,34 +243,29 @@ double spline2(double x);                                                  //  r
 
 //  application initialization and administration
 
-#define ZTXmaxent 1000                                                     //  max. translation strings
-#define ZTXmaxcc 2000                                                      //  max. cc per string
+#define ZTXmaxent 2000                                                     //  max. translation strings
+#define ZTXmaxcc 4000                                                      //  max. cc per string
 
-int initz_appfiles(cchar *appname, ...);                                   //  find app dirs, copy initial files
+int zinitapp(cchar *appname, ...);                                         //  initz. app directories and files   v.4.1
+cchar * get_zprefix();                                                     //  get /usr or /usr/local  ...        v.4.1
 cchar * get_zuserdir();                                                    //  get /home/user/.appname/
-cchar * get_zdatadir();                                                    //  get main install directory
-void showz_userguide(cchar *context = 0);                                  //  show help file in new process
+cchar * get_zdatadir();                                                    //  get install directory
+cchar * get_zdocdir();                                                     //  get document directory             v.3.8
+
+void showz_userguide(cchar *context = 0);                                  //  show user guide in new process
 void showz_readme();                                                       //  show README file in popup window
 void showz_changelog();                                                    //  show changelog file in popup window
 void showz_translations();                                                 //  show TRANSLATIONS file in popup window
-void zmake_launcher(cchar *categories, cchar *genericname);                //  create desktop icon/launcher   v.2.18
-
-int ZTXinit(cchar *lang);                                                  //  setup for message translation
-cchar *ZTX(cchar *english);                                                //  get translation for English message
+void showz_doctext(cchar *file);                                           //  show text file (or .gz) in a popup window
+void zmake_menu_launcher(cchar *command, cchar *cats, cchar *generic);     //  add desktop menu and launcher      v.4.1
+void showz_html(cchar *url);                                               //  show html via preferred browser  v.2.18
 
 /**************************************************************************
    GTK utility functions
 ***************************************************************************/
 
-#include <gtk/gtk.h>
-#include <gdk/gdkkeysyms.h>
-
-#define  zdcbmax 100                                                       //  max. combo box drop-down list
-
-void zlockInit();                                                          //  lock GTK for threaded applications
-void zlock();  
-void zunlock();
 void zmainloop(int skip = 0);                                              //  do main loop, process menu events
+void zthreadcrash();                                                       //  crash if thread is not main() thread
 
 //  text window print and read utilities
 void wprintx(GtkWidget *Win, int line, cchar *mess, cchar *font = 0);      //  write text to line, optional font
@@ -262,7 +275,7 @@ void wscroll(GtkWidget *mLog, int line);                                   //  s
 void wclear(GtkWidget *Win);                                               //  clear window
 void wclear(GtkWidget *Win, int line);                                     //  clear from line to end
 char * wscanf(GtkWidget *Win, int &ftf);                                   //  get text lines from edit widget
-int wfiledump(GtkWidget *Win, char *filespec);                             //  dump text window to file
+int  wfiledump(GtkWidget *Win, char *filespec);                            //  dump text window to file
 void wfilesave(GtkWidget *Win);                                            //  wfiledump() via file-chooser dialog
 void wprintp(GtkWidget *Win);                                              //  print text window to default printer
 
@@ -270,15 +283,25 @@ void wprintp(GtkWidget *Win);                                              //  p
 
 //  functions to simplify building menus, tool bars, status bars
 
-#define G_SIGNAL(window,event,func,arg) g_signal_connect(G_OBJECT(window),event,G_CALLBACK(func),(void *) arg);
+#define G_SIGNAL(window,event,func,arg) \
+        g_signal_connect(G_OBJECT(window),event,G_CALLBACK(func),(void *) arg)
+
+#define  zdcbmax 100                                                             //  max. combo box drop-down list
 
 typedef void mtFunc(GtkWidget *, cchar *mname);                                  //  menu or button response function
+
 GtkWidget * create_menubar(GtkWidget *vbox);                                     //  create menubar in packing box
 GtkWidget * add_menubar_item(GtkWidget *mbar, cchar *mname, mtFunc func = 0);    //  add menu item to menubar 
 GtkWidget * add_submenu_item(GtkWidget *mitem, cchar *label, mtFunc func = 0);   //  add submenu item to menu item 
-GtkWidget * create_toolbar(GtkWidget *vbox, int iconsize = 24, int vert = 0);    //  horiz. toolbar in packing box (vert.)
+
+GtkWidget * create_popmenu();                                                    //  create an empty popup menu
+GtkWidget * add_popmenu_item(GtkWidget *popmenu, cchar *mname, mtFunc func);     //  add menu item to popup menu
+void popup_menu(GtkWidget *popmenu);                                             //  pop-up menu at current mouse position
+
+GtkWidget * create_toolbar(GtkWidget *vbox, int iconsize = 24);                  //  toolbar in packing box (no vert gtk3)
 GtkWidget * add_toolbar_button(GtkWidget *tbar, cchar *label, cchar *tip,        //  add button with stock (gtk-quit) 
                                  cchar *icon, mtFunc func);                      //    or custom icon (iconfile.png)
+
 GtkWidget * create_stbar(GtkWidget *vbox);                                       //  create status bar in packing box
 int stbar_message(GtkWidget *stbar, cchar *message);                             //  display message in status bar
 
@@ -302,6 +325,8 @@ struct zwidget  {
       int         homog;                           //  hbox/vbox: equal spacing flag
       int         expand;                          //  expandable flag
       int         space;                           //  extra padding space (pixels)
+      int         wrap;                            //  wrap mode for edit widget
+      int         stopKB;                          //  stop propagation of KB events to parent
       GtkWidget   *widget;                         //  GTK widget pointer
 };
 
@@ -310,6 +335,10 @@ struct zdialog  {
       void        *eventCB;                        //  widget event user callback function
       int         zstat;                           //  dialog status (from completion button)
       int         disabled;                        //  widget signals/events are disabled
+      int         saveposn;                        //  save and recall window position each use
+      int         stopKB;                          //  next KB event will be ignored
+      cchar       *help_topic;                     //  optional help topic
+      GtkWidget   *parent;                         //  parent window or null
       GtkWidget   *compbutt[zdmaxbutts];           //  dialog completion buttons
       zwidget     widget[zdmaxwidgets];            //  dialog widgets (EOF = type = 0)
 };
@@ -319,27 +348,30 @@ zdialog *zdialog_new(cchar *title, GtkWidget *parent, ...);                //  c
 int zdialog_add_widget(zdialog *zd,                                        //  add widget to zdialog
       cchar *type, cchar *name, cchar *pname,                              //  required args
       cchar *data = 0, int scc = 0, int homog = 0,                         //  optional args
-      int expand = 0, int space = 0);
+      int expand = 0, int space = 0, int wrap = 0);
 
 int zdialog_add_widget(zdialog *zd,                                        //  add widget to zdialog
       cchar *type, cchar *name, cchar *pname,                              //  (alternative form)
-      cchar *data, cchar *options);                                        //  "scc=nn|homog|expand|space=nn"
+      cchar *data, cchar *options);                                        //  "scc=nn|homog|expand|space=nn|wrap"
 
 GtkWidget * zdialog_widget(zdialog *zd, cchar *name);                      //  GTK widget from zdialog widget name
 int zdialog_set_group(zdialog *zd, cchar *radio1, ...);                    //  set group ID for set of radio buttons
 int zdialog_resize(zdialog *zd, int width, int height);                    //  set size > widget sizes
 int zdialog_put_data(zdialog *zd, cchar *name, cchar *data);               //  put data in widget (entry, spin ...)
 cchar * zdialog_get_data(zdialog *zd, cchar *name);                        //  get widget data
+int zdialog_set_limits(zdialog *zd, cchar *name, double min, double max);  //  set new widget limits (spin, scale)
+void zdialog_help(zdialog *zd, cchar *help_topic);                         //  set help topic for zdialog
 
 typedef int zdialog_event(zdialog *zd, cchar *name);                       //  widget event callback function
-int zdialog_run(zdialog *zd, zdialog_event = 0);                           //  run dialog, handle events
+int zdialog_run(zdialog *zd, zdialog_event = 0, cchar *posn = 0);          //  run dialog, handle events
 
 int zdialog_send_event(zdialog *zd, cchar *event);                         //  send an event to an active dialog   v.2.17
 int zdialog_send_response(zdialog *zd, int zstat);                         //  complete a dialog, set status   v.2.23
 int zdialog_destroy(zdialog *zd);                                          //  destroy dialog (caller responsibility)
-int zdialog_free(zdialog *zd);                                             //  free zdialog memory
+int zdialog_free(zdialog *&zd);                                            //  free zdialog memory
 int zdialog_wait(zdialog *zd);                                             //  wait for dialog status or be destroyed
 int zdialog_goto(zdialog *zd, cchar *name);                                //  put cursor at named widget  v.2.23
+void zdialog_set_cursor(zdialog *zd, GdkCursor *cursor);                   //  set cursor for dialog window
 
 int zdialog_stuff(zdialog *zd, cchar *name, cchar *data);                  //  stuff string data into widget
 int zdialog_stuff(zdialog *zd, cchar *name, int  data);                    //  stuff int data
@@ -355,46 +387,58 @@ int zdialog_cb_prep(zdialog *zd, cchar *name, cchar *data);                //  p
 char * zdialog_cb_get(zdialog *zd, cchar *name, int Nth);                  //  get combo drop down list Nth entry
 int zdialog_cb_delete(zdialog *zd, cchar *name, cchar *data);              //  delete combo drop down list entry   v.2.4
 int zdialog_cb_clear(zdialog *zd, cchar *name);                            //  clear all combo box entries         v.2.4
+int zdialog_cb_popup(zdialog *zd, cchar *name);                            //  show all combo box list entries     v.3.8
+
+int  zdialog_positions(cchar *action);                                     //  load or save zdialog window positions
+void zdialog_set_position(zdialog *zd, cchar *posn);                       //  set initial/new zdialog window position
+void zdialog_save_position(zdialog *zd);                                   //  save zdialog window position
 
 /**************************************************************************/
 
-//   GTK misc. utility functioins
+//  translation functions
 
-int write_popup_text(cchar *action, cchar *text = 0, int ww = 0, int hh = 0, GtkWidget *parent = 0);  //  write text to popup window
-int popup_command(cchar *command, int ww = 400, int hh = 300, GtkWidget *parent = 0);                 //  shell command to popup window
+void ZTXinit(cchar *lang);                                                 //  setup for message translation
+cchar *ZTX(cchar *english);                                                //  get translation for English message
+void ZTX_translation_start(GtkWidget *parent);                             //  start online translation
 
-void zmessageACK(GtkWindow *, cchar *pMess, ... );                         //  display message, wait for OK
-void zmessLogACK(GtkWindow *, cchar *pMess, ... );                         //  same, with log to STDOUT
-int zmessageYN(GtkWindow *, cchar *pMess, ... );                           //  display message, wait for YES or NO
-GtkWidget * zmessage_post(GtkWindow *, cchar *pMess, ...);                 //  display message indefinitely   v.2.6
-void zmessage_kill(GtkWidget *dialog);                                     //  kill zmessage_post() message
+//  write text to popup window, shell command to popup window
 
-cchar * dialogText(GtkWindow *, cchar * title, cchar * inittext);          //  get short text input from user
+int write_popup_text(cchar *action, cchar *text = 0, int ww = 0, int hh = 0, GtkWidget *parent = 0);
+int popup_command(cchar *command, int ww = 400, int hh = 300, GtkWidget *parent = 0);
 
-char * zgetfile(cchar *title, cchar *file, cchar *rw, cchar *butt = 0);    //  file chooser dialog       v.2.3
-char ** zgetfiles(cchar *title, cchar *file, cchar *butt = 0);             //  multiple file select dialog     v.2.29
+//  popup message dialogs
 
-void showz_html(cchar *url);                                               //  show html via preferred browser  v.2.18
+void zmessageACK(GtkWidget *parent, cchar *pMess, ... );                   //  display message, wait for OK
+void zmessLogACK(GtkWidget *parent, cchar *pMess, ... );                   //  same, with log to STDOUT
+int zmessageYN(GtkWidget *parent, cchar *pMess, ... );                     //  display message, wait for YES or NO
+void zmessage_help(GtkWidget *parent, cchar *topic, cchar *mess, ...);     //  message box, [help] > user guide topic
+zdialog * zmessage_post(GtkWidget *parent, int secs, cchar *mess, ...);    //  display message until timeout or cancel
+char * zdialog_text(GtkWidget *parent, cchar * title, cchar * initext);    //  get short text input from user
+int zdialog_choose(cchar *title, GtkWidget *parent, cchar *mess, ...);     //  show message and return choice button
+
+//  file chooser dialogs for one file or multiple files
+
+char  * zgetfile1(cchar *title, cchar *action, cchar *file, cchar *butt = 0);
+char ** zgetfileN(cchar *title, cchar *action, cchar *file, cchar *butt = 0);
+
+//  print image file
+
+void print_image_paper_setup(GtkWidget *parent);                           //  paper setup for printing an image file
+void print_image_margins_setup(GtkWidget *parent);                         //  setup print margins
+void print_image_file(cchar *imagefile);                                   //  print the image file
+
+//  drag and drop functions
 
 typedef void drag_drop_func(int x, int y, char *text);                     //  user function, get drag_drop text  v.2.19
 void drag_drop_connect(GtkWidget *window, drag_drop_func);                 //  connect window to user function
 
-/**************************************************************************/
+//  miscellaneous GDK/GTK functions
 
-//   GDK/GTK image file utility functions
-//   for functions returning char *, caller responsible for zfree()
-
-void print_imagefile(cchar *file, cchar *format = 0, cchar *orien = 0);    //  print image file (.jpg .png .tif)  v.3.4
+GdkPixbuf * get_thumbnail(char *fpath, int size);                          //  get sized thumbnail for image file
+GdkCursor * zmakecursor(cchar *iconfile);                                  //  make a cursor from an image file   v.3.7
+int gdk_window_move_pointer(GdkWindow *, int px, int py);                  //  move the mouse pointer to px, py
 GdkPixbuf * gdk_pixbuf_rotate(GdkPixbuf *, double deg, int alfa = 0);      //  rotate pixbuf through any angle
 
-typedef void image_cbfunc(char *file);                                     //  callback function for clicked thumbnail
-char * image_gallery(cchar *filez, cchar *action, int Nth = 1,             //  display image gallery window, navigate,
-                            image_cbfunc ufunc = 0);                       //    do callback for clicked thumbnails
-void image_position(cchar *file, int &posn, int &count);                   //  get curr. image position, total image count
-int image_file_type(cchar *file);                                          //  determine if directory or image file type
-char * image_thumbfile(cchar *imagefile);                                  //  get thumbnail filespec, create if missing
-GdkPixbuf * image_thumbnail(cchar *imagefile, int size = 0);               //  get thumbnail pixbuf, create if missing
-
 /**************************************************************************/
 
 //  parameter management functions
